import functools
from abc import ABC, abstractmethod
from inspect import signature, Parameter
from typing import Callable

from cpl_core.dependency_injection import ServiceProviderABC
from cpl_core.type import R, T
from cpl_query.extension import List


class FilterABC(ABC):
    def __init__(self):
        ABC.__init__(self)

    @abstractmethod
    def from_dict(self, values: dict):
        pass

    @abstractmethod
    def filter(self, query: List, *args) -> List:
        pass

    @staticmethod
    @ServiceProviderABC.inject
    def get_filter(f, values: dict, services: ServiceProviderABC):
        sig = signature(f)
        for param in sig.parameters.items():
            parameter = param[1]
            if parameter.name == "self" or parameter.name == "cls" or parameter.annotation == Parameter.empty:
                continue

            if issubclass(parameter.annotation, FilterABC):
                filter = services.get_service(parameter.annotation)
                filter.from_dict(values)
                return filter @ staticmethod

    @staticmethod
    @ServiceProviderABC.inject
    def get_collection_filter(filter_type: type, values: dict, services: ServiceProviderABC):
        filter: FilterABC = services.get_service(filter_type)
        filter.from_dict(values)
        return filter

    @classmethod
    def resolve_filter_annotation(cls, f=None):
        if f is None:
            return functools.partial(cls.resolve_filter_annotation)

        @functools.wraps(f)
        def decorator(*args, **kwargs):
            if "filter" in kwargs:
                kwargs["filter"] = cls.get_filter(f, kwargs["filter"])

            return f(*args, **kwargs)

        return decorator

    def _get_attributes_from_dict(self, attrs: list[tuple[str, type]], values: dict):
        for key_pair in attrs:
            attr = key_pair[0]
            attr_type = key_pair[1]
            if attr in values:
                self.__setattr__(f"_{attr}", attr_type(values[attr]))

    @staticmethod
    def _filter_by_attributes(attrs: list[dict], values: List[T]) -> List[R]:
        for attr in attrs:
            if attr["attr"] is None:
                continue
            values = values.where(attr["func"])
        return values
